package org.kapelle.multiblock;

import org.bukkit.Location;
import org.bukkit.util.BlockVector;

import java.util.List;

/**
 * Represents a full multiblock.
 * Contains the individual block components.
 */
public class MultiblockStructure {

    List<List<List<BlockComponent>>> structure;

    /**
     * Create a multiblock structure.
     * The format is like this: [y][z][x]
     * So you describe the layers from bottom to top.
     * @param structure structure in the format: [y][z][x]
     */
    public MultiblockStructure(List<List<List<BlockComponent>>> structure){
        this.structure = structure;
    }

    public BlockComponent getComponent(BlockVector offset){
        return getComponent(offset.getBlockX(),offset.getBlockY(),offset.getBlockZ());
    }

    public BlockComponent getComponent(int x, int y, int z){
        return structure.get(y).get(z).get(x);
    }

    /**
     * Checks if the multiblock is valid at the given location.
     * @param location Location of a block in a multiblock structure.
     * @param offset the offset of the location from <0,0,0>
     * @return true if valid
     */
    public boolean isValid(Location location, BlockVector offset){
        for (int y = 0; y < structure.size(); y++) {
            for (int z = 0; z < structure.get(y).size(); z++) {
                for (int x = 0; x < structure.get(y).get(z).size() ; x++) {

                    BlockComponent currentComponent = getComponent(x,y,z);
                    Location globalBlockLocation = location.clone().subtract(offset).add(new BlockVector(x,y,z));

                    if(!currentComponent.isValidBlock(globalBlockLocation)){
                        return false;
                    }
                }
            }
        }

        return true;
    }
}
